"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Reference_1 = require("./coordinate/Reference");
const Range_1 = require("./coordinate/Range");
const Sheet_1 = require("./coordinate/axis/Sheet");
const debug_1 = require("./debug");
const { Parser: FormulaParser } = require('hot-formula-parser');
function createReference(cell, defaultSheet) {
    return Reference_1.default.from(`${cell.sheet || defaultSheet.index.base1}!${cell.column.label}${cell.row.label}`);
}
class Formula {
    constructor(formula, sheet) {
        this.formula = formula;
        this.sheet = sheet;
    }
    getMembers(lookUp) {
        debug_1.debug('Formula#getOperandCells()');
        let errs = [];
        let cells = [];
        const parser = new FormulaParser();
        parser.on('callCellValue', (cell, done) => {
            try {
                debug_1.debug('Formula#getOperandCells(): event "callCellValue" [%s!%s%s]', cell.sheet, cell.column.label, cell.row.label);
                const ref = createReference(cell, this.sheet);
                debug_1.debug('Formula#getOperandCells():   => %o', ref.toString());
                cells.push(ref);
                lookUp(ref, (err, result) => {
                    if (err) {
                        errs.push(err);
                        return;
                    }
                    done(result);
                });
            }
            catch (e) {
                errs.push(e);
                throw e;
            }
        });
        parser.on('callRangeValue', (start, end, done) => {
            try {
                debug_1.debug('Formula#getOperandCells(): event "callRangeValue" [%s!%s%s] => [%s!%s%s]', start.sheet, start.column.label, start.row.label, end.sheet, end.column.label, end.row.label);
                const table = new Range_1.default(createReference(start, this.sheet), createReference(end, this.sheet)).toArray();
                debug_1.debug('Formula#getOperandCells():   => %o', table.map(ref => ref.toString()));
                cells = cells.concat(table.reduce((acc, row) => {
                    acc.push(...row);
                    return acc;
                }, []));
                done(table.map(row => row.map(ref => {
                    let v;
                    lookUp(ref, (err, result) => {
                        if (err) {
                            errs.push(err);
                            return;
                        }
                        v = result;
                    });
                    return v;
                })));
            }
            catch (e) {
                errs.push(e);
                throw e;
            }
        });
        parser.parse(this.formula);
        if (errs.length > 0)
            throw errs[0];
        return cells;
    }
    valueOf() {
        return this.formula;
    }
    toString() {
        return `=${this.formula}`;
    }
    static from(formula, sheet) {
        return new Formula(formula.replace(/^=/, ''), typeof sheet === 'string'
            ? Sheet_1.default.from(sheet)
            : sheet);
    }
    static isFormula(literal) {
        return literal.startsWith('=');
    }
}
exports.Formula = Formula;
