"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = require("./debug");
const Formula_1 = require("./Formula");
async function buildContext(entries, lookUp, context = {}) {
    debug_1.debug('buildContext(): entry point = %o', entries.map(ref => ref.label));
    // debug('buildContext(): initial context = %o', context)
    for (let cell of entries) {
        await new Promise(function (resolve, reject) {
            lookUp(cell, async (err, value) => {
                if (err)
                    return reject(err);
                debug_1.debug('buildContext() -> lookUp: cell "%s" = %o', cell.label, value);
                try {
                    context[cell.toString()] = value;
                    if (typeof value === 'string' && Formula_1.Formula.isFormula(value)) {
                        debug_1.debug('buildContext(): formula %o', value);
                        const formula = Formula_1.Formula.from(value, cell.sheet);
                        const operands = formula.getMembers((ref, done) => {
                            lookUp(ref, (err, v, result) => {
                                if (err) {
                                    done(err, null);
                                    return;
                                }
                                done(null, result);
                            });
                        });
                        debug_1.debug('buildContext(): operand cells = %o', operands);
                        if (operands.length > 0) {
                            Object.assign(context, await buildContext(operands, lookUp, context));
                            // debug('buildContext(): new context = %o', context)
                        }
                    }
                }
                catch (err) {
                    return reject(err);
                }
                resolve();
            });
        });
    }
    return context;
}
exports.default = buildContext;
