"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const base26 = require("base26");
class Axis {
    valueOf() {
        return this.index;
    }
    toString() {
        return this.label;
    }
}
class Row extends Axis {
    constructor(label) {
        super();
        if (typeof label === 'string') {
            this.abs = label.startsWith('$');
            this.index = Number(label.replace(/^\$/, '')) - 1;
            this.label = label;
        }
        else {
            this.abs = true;
            this.index = label;
            this.label = '$' + (this.index + 1);
        }
    }
}
exports.Row = Row;
class Column extends Axis {
    constructor(labelOrIndex) {
        super();
        this.index = 0;
        if (typeof labelOrIndex === 'string') {
            this.label = labelOrIndex.toUpperCase();
            this.abs = labelOrIndex.startsWith('$');
            this.index = base26.from(this.label.replace(/^\$/, '').toLowerCase()) - 1;
        }
        else {
            this.label = '$' + base26.to(labelOrIndex + 1).toUpperCase();
            this.index = labelOrIndex;
            this.abs = true;
        }
    }
}
exports.Column = Column;
class InvalidCoordError extends Error {
    constructor(text) {
        super(`Invalid coordinate is given: "${text}".`);
        this.name = 'InvalidCoordError';
    }
}
exports.InvalidCoordError = InvalidCoordError;
class Coordinate {
    constructor(arg1, row, sheet = 1) {
        this.sheet = 1;
        if (typeof arg1 === 'string') {
            let matched = arg1.match(/(?:(\d+)!)?(\$?[A-Z]+)(\$?\d+)/);
            if (!matched)
                throw new InvalidCoordError(arg1);
            this.sheet = Number(matched[1] || 1);
            this.column = new Column(matched[2]);
            this.row = new Row(matched[3]);
        }
        else {
            this.column = arg1;
            this.row = row;
            this.sheet = sheet;
        }
        this.label = this.column.toString() + this.row.toString();
    }
    shift(deltaCol = 0, deltaRow = 0) {
        return new Coordinate(new Column(this.column.index + deltaCol), new Row(this.row.index + deltaRow), this.sheet);
    }
    isAbsolute() {
        return this.column.abs && this.row.abs;
    }
    valueOf() {
        return [
            this.column.index,
            this.row.index
        ];
    }
    toJSON() {
        return {
            label: this.label,
            sheet: this.sheet
        };
    }
    toString() {
        return this.sheet + '!' + this.label;
    }
}
exports.Coordinate = Coordinate;
class CoordRelation {
    constructor(rel, pivot) {
        this.rel = rel;
        this.pivot = pivot;
        this.offset = [0, 0];
        if (!this.pivot.isAbsolute())
            throw new TypeError(`The pivot should be absolute.`);
        this.offset = [
            this.rel.column.index - this.pivot.column.index,
            this.rel.row.index - this.pivot.row.index
        ];
    }
    resolveFrom(target) {
        if (!target.isAbsolute())
            throw new TypeError(`The target should be absolute.`);
        return target.shift(this.offset[0], this.offset[1]);
    }
}
exports.CoordRelation = CoordRelation;
