"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Reference_1 = require("./Reference");
const CrossSheetRangeError_1 = require("../errors/CrossSheetRangeError");
const Vector_1 = require("./Vector");
class Range {
    constructor(start, end) {
        this.start = start;
        this.end = end;
        if (!this.start.sheet.equals(this.end.sheet))
            throw new CrossSheetRangeError_1.default();
    }
    toArray() {
        const unitXVector = new Vector_1.default(1, 0);
        const unitYVector = new Vector_1.default(0, 1);
        const table = [];
        let curRef = this.start;
        for (; curRef.row.index.base0 <= this.end.row.index.base0; curRef = curRef.offset(unitYVector)) {
            const row = [];
            for (; curRef.column.index.base0 <= this.end.column.index.base0; curRef = curRef.offset(unitXVector)) {
                row.push(curRef);
            }
            // because curRef.column.index.base0 > this.end.column.index.base0
            // so the for loop right above breaks
            // curRef should be reset in x-axis
            curRef = curRef.offset(new Vector_1.default(-1 * (this.end.column.index.base0 - this.start.column.index.base0 + 1), 0));
            table.push(row);
        }
        return table;
    }
    get label() {
        return this.start.label + ':' + this.end.column.label + this.end.row.label;
    }
    static from(start, end) {
        return new Range(Reference_1.default.from(start), Reference_1.default.from(end));
    }
}
exports.default = Range;
