"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Column_1 = require("./axis/Column");
const Row_1 = require("./axis/Row");
const Sheet_1 = require("./axis/Sheet");
const ValueError_1 = require("../errors/ValueError");
const Index_1 = require("./axis/Index");
/**
 * Reference is the absolute cell reference used in excel
 */
class Reference {
    constructor(sheet, column, row) {
        this.sheet = sheet;
        this.column = column;
        this.row = row;
    }
    offset(vec) {
        return new Reference(this.sheet, new Column_1.default(new Index_1.default(this.column.index.base0 + vec.colOffset, 0)), new Row_1.default(new Index_1.default(this.row.index.base0 + vec.rowOffset, 0)));
    }
    equals(another) {
        return this.column.equals(another.column) &&
            this.row.equals(another.row) &&
            this.sheet.equals(another.sheet);
    }
    toString() {
        return `${this.sheet.toString()}${this.column.toString()}${this.row.toString()}`;
    }
    get label() {
        return `${this.column.toString()}${this.row.toString()}`;
    }
    /**
     * @param label Omitting "$" is allowed but every cell ref is treated as an absolute ref.
     */
    static from(label) {
        const matched = label.match(/(\d+)!\$?([a-zA-Z]+)\$?(\d+)/);
        if (!matched)
            throw new ValueError_1.default(`The ref label ("${label}") is invalid.`);
        return new Reference(Sheet_1.default.from(matched[1]), Column_1.default.from(matched[2]), Row_1.default.from(matched[3]));
    }
}
exports.default = Reference;
