"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const context = {};
/**
 * Context always has the coordinates in absolute format.
 */
function evalFormula(entry, args, ...evalArgs) {
    const FormulaParser = require('hot-formula-parser').Parser;
    const parser = new FormulaParser();
    // /***** Common Methods *****/
    function parse(expr) {
        if (typeof expr === 'string' && expr.startsWith('=')) {
            const { error, result } = parser.parse(expr.replace(/^=/, ''));
            if (error) {
                throw createError('FormulaError', error);
            }
            return result;
        }
        else {
            return expr;
        }
    }
    function createError(name, msg) {
        const e = new Error(msg);
        e.name = name;
        return e;
    }
    /**
     * @see https://github.com/jameslaneconkling/base26
     */
    function toBase26(decimal) {
        let out = '';
        while (true) {
            out = String.fromCharCode(((decimal) % 26) + 97) + out;
            decimal = Math.floor((decimal) / 26);
            if (decimal === 0) {
                break;
            }
        }
        return out;
    }
    function parseRange(start, end) {
        const rety = [];
        for (let y = start.row.index; y <= end.row.index; y++) {
            const retx = [];
            for (let x = start.column.index; x <= end.column.index; x++) {
                retx.push(`${start.sheet}!$${toBase26(x).toUpperCase()}$${y + 1}`);
            }
            rety.push(retx);
        }
        return rety;
    }
    // /**************************/
    const localContext = Object.assign({}, context);
    // context replacement
    if (Array.isArray(args)) {
        args.forEach((arg, i) => {
            localContext[arg] = evalArgs[i] === undefined ? context[arg] : evalArgs[i];
        });
    }
    const errors = [];
    parser.on('callCellValue', function ({ row: { label: rowLabel }, column: { label: colLabel }, sheet }, done) {
        try {
            done(parse(localContext[`${sheet}!$${colLabel}$${rowLabel}`]));
        }
        catch (e) {
            errors.push(e);
            throw e;
        }
    });
    parser.on('callRangeValue', function (start, end, done) {
        try {
            done(parseRange(start, end).map((labels) => labels.map(label => parse(localContext[label]))));
        }
        catch (e) {
            errors.push(e);
            throw e;
        }
    });
    if (!entry) {
        throw createError('EntryError', 'The entry cell is not defined.');
    }
    const result = parse(localContext[entry]);
    if (errors.length > 0)
        throw errors[0];
    return result;
}
exports.evalFormula = evalFormula;
